import { Component, OnInit, ViewChild, ElementRef } from '@angular/core';
import { Aeropuerto } from '../../interfaces/aeropuertos.interfaces';
import { Router } from '@angular/router';
import Swal from 'sweetalert2';
import { AeropuertosService } from '../../services/aeropuertos.service';
import {MatPaginator} from '@angular/material/paginator';
import {MatSort} from '@angular/material/sort';
import {MatTableDataSource} from '@angular/material/table'

@Component({
  selector: 'app-listar',
  templateUrl: './listar.component.html',
  styleUrls: ['./listar.component.css']
})
export class ListarComponent implements OnInit{

  displayedColumns: string[] = ['codigo', 'nombre','opciones'];
  dataSource!: MatTableDataSource<any>;
  @ViewChild(MatSort) sort!: MatSort;
  @ViewChild(MatPaginator) paginator!: MatPaginator;
  @ViewChild('input') entradaFiltro: ElementRef | undefined;
  public filtroInput: string = '';  
  public aeropuertos: Aeropuerto[] = [];

  constructor(
    private router: Router,
    private aeropuertosService: AeropuertosService
  ){}

  ngOnInit(): void {
      setTimeout(() => {
        this.listaAeropuertos();
      }, 200);
  }

  listaAeropuertos(){
    this.aeropuertosService.getAeropuertos().subscribe(
      res =>{
        this.aeropuertos = res;
        this.dataSource = new MatTableDataSource(this.aeropuertos);
        this.dataSource.paginator = this.paginator;  
        this.dataSource.sort = this.sort;
      },err => {

      }
    )
  }


  eliminar(event:any, id: number){
    event.stopPropagation();

    Swal.fire({
      title: 'Estas seguro?' + ' Aeropuerto ID: ' + id,
      text: "No se podran revertir los cambios!",
      icon: 'warning',
      showCancelButton: true,
      confirmButtonColor: '#3085d6',
      cancelButtonColor: '#d33',
      confirmButtonText: 'Si, eliminar!',
      cancelButtonText: 'Cancelar'
    }).then((result) => {
      if (result.isConfirmed) {

        this.eliminarAeropuerto(id);

        const Toast = Swal.mixin({
          toast: true,
          position: 'top-end',
          showConfirmButton: false,
          timer: 2000,
          timerProgressBar: true,
          didOpen: (toast) => {
            toast.addEventListener('mouseenter', Swal.stopTimer)
            toast.addEventListener('mouseleave', Swal.resumeTimer)
          }
        })
        
        Toast.fire({
          icon: 'success',
          title: 'Se elimino correctamente'
        })
      }
    })
  }

  detalles(id:number){
    this.router.navigate(['/aeropuertos/detalles/',id ]);
  }
  actualizar(id: number){
    this.router.navigate(['/aeropuertos/editar/',id ]);
  }
  
  eliminarAeropuerto(id: number){
    this.aeropuertosService.deleteAeropuerto(id).subscribe(
      res =>{
        this.listaAeropuertos();
      },err => {

      }
    )
  }
  
  aplicarFiltro(event: Event) {
    const filterValue = this.filtroInput;
    this.dataSource.filter = filterValue.trim().toLowerCase();

    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  eliminarFiltros(){
    this.filtroInput = '';
    this.dataSource.filter = '';
    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }
}
